@@ -11,9 +11,7 @@ import zipfile
 from tqdm import tqdm
 from tqdm.utils import CallbackIOWrapper
 
-from .utils import zipdir, getApiKey, generate_requirements_file, auth, get_model, new_version, deploy_version, find_version
-
-api_host = 'https://api.inferrd.com'
+from .utils import zipdir, getApiKey, run_job, new_job, get_api_host, generate_requirements_file, auth, set_api_host,  get_model, new_version, deploy_version, find_version
 
 settings = EasySettings(str(Path.home()) + "/.inferrd.conf")
 
@@ -22,8 +20,16 @@ __all__ = [
     'doctools'
 ]
 
-def __main__():
-  print('Hi')
+def __set_api_host(host):
+  set_api_host(host)
+  print('API Endpoint set to ' + host)
+
+def sizeof_fmt(num, suffix='B'):
+  for unit in ['','Ki','Mi','Gi','Ti','Pi','Ei','Zi']:
+    if abs(num) < 1024.0:
+      return "%3.1f%s%s" % (num, unit, suffix)
+    num /= 1024.0
+  return "%.1f%s%s" % (num, 'Yi', suffix)
 
 # arguments
 # includeFailures=True/False
@@ -49,12 +55,79 @@ def get_requests(name, **kwargs):
     v = find_version(model['id'], kwargs['version'])
     version = v['id']
 
-  url = api_host + '/service/' + model['id'] + '/requests?' + ('responseStatus=200&' if not includeFailures else '') + 'limit=' + str(limit) + '&page=' + str(page) + ('&version=' + version if version else '')
+  url = get_api_host() + '/service/' + model['id'] + '/requests?' + ('responseStatus=200&' if not includeFailures else '') + 'limit=' + str(limit) + '&page=' + str(page) + ('&version=' + version if version else '')
 
   r = requests.get(url, headers={'Content-Type': 'application/json', 'Authorization': 'Token ' + api_key})
 
   return r.json()
 
+def run(team_name, job, **kwargs):
+  if(getApiKey() == ''):
+    raise Exception('No api key. Use inferrd.auth() first.')
+
+  if 'name' not in kwargs:
+    raise Exception('Missing argument "name". Your job needs a name.')
+
+  gpu_ram = 2000
+  cpu_ram = 4000
+
+  if 'cpu_ram' in kwargs:
+    cpu_ram = kwargs['cpu_ram']
+
+  if 'gpu_ram' in kwargs:
+    gpu_ram = kwargs['gpu_ram']
+  
+  print('> Packaging your job')
+
+  if os.path.exists('./job.dill'):
+    os.remove('./job.dill')
+
+  dill.dump(job, open('./job.dill', mode='wb'), recurse=True)
+
+  if os.path.exists('./reqs.txt'):
+    os.remove('./reqs.txt')
+
+  generate_requirements_file()
+
+  print('> Zipping job for upload')
+
+  zipf = zipfile.ZipFile('job.zip', 'w', zipfile.ZIP_DEFLATED)
+  zipf.write('./job.dill', './job.dill')
+  zipf.write('./reqs.txt', './requirements.txt')
+
+  if 'setup' in kwargs:
+    setupFile = open('./setup.sh', 'w')
+    setupFile.write(kwargs['setup'])
+    setupFile.close()
+    zipf.write('./setup.sh', './setup.sh')
+
+  zipf.close()
+
+  job = new_job(
+    kwargs['name'],
+    team_name,
+    gpu_ram=gpu_ram,
+    cpu_ram=cpu_ram
+  )
+
+  model_size = Path('./job.zip').stat().st_size
+  print('> Uploading job ({0})'.format(sizeof_fmt(model_size)), flush=True)
+  f = open("./job.zip", 'rb')
+
+  with tqdm(total=model_size, unit="B", unit_scale=True, unit_divisor=1024, ncols=100) as t:
+    wrapped_file = CallbackIOWrapper(t.update, f, "read")
+    r = requests.put(job['signedUpload'], data=wrapped_file, headers={'Content-Type': 'application/zip'})
+
+  print('> Starting job ' + job['name'])
+  run_job(job['id'])
+
+  os.remove('./job.zip')
+  os.remove('./job.dill')
+  os.remove('./reqs.txt')
+
+  if 'setup' in kwargs:
+    os.remove('./setup.sh')
+
 def deploy(model, **kwargs):
   if(getApiKey() == ''):
     raise Exception('No api key. Use inferrd.auth() first.')
@@ -69,7 +142,7 @@ def deploy(model, **kwargs):
 
   version = new_version(model['id'])
 
-  print('> Preping model for deployment')
+  print('> Packaging your model for deployment')
 
   if os.path.exists('./model.dill'):
     os.remove('./model.dill')
@@ -81,6 +154,8 @@ def deploy(model, **kwargs):
 
   generate_requirements_file()
 
+  print('> Zipping model for upload')
+
   zipf = zipfile.ZipFile('model.zip', 'w', zipfile.ZIP_DEFLATED)
   zipf.write('./model.dill', './model.dill')
   zipf.write('./reqs.txt', './requirements.txt')
@@ -95,7 +170,7 @@ def deploy(model, **kwargs):
   
   # upload to storage
   model_size = Path('./model.zip').stat().st_size
-  print('> Uploading model ({0}MB)'.format(round(model_size / 1024 / 1024, 3)))
+  print('> Uploading model ({0})'.format(sizeof_fmt(model_size)), flush=True)
   f = open("./model.zip", 'rb')
 
   with tqdm(total=model_size, unit="B", unit_scale=True, unit_divisor=1024, ncols=100) as t:
@@ -147,9 +222,13 @@ def deploy_tf(tf_model, name):
   zipf.close()
 
   # upload to storage
-  print('> Uploading model')
+  model_size = Path('./model.zip').stat().st_size
+  print('> Uploading model ({0})'.format(sizeof_fmt(model_size)), flush=True)
   f = open("./model.zip", 'rb')
-  r = requests.put(version['signedUpload'], data=f, headers={'Content-Type': 'application/zip'})
+  
+  with tqdm(total=model_size, unit="B", unit_scale=True, unit_divisor=1024, ncols=100) as t:
+    wrapped_file = CallbackIOWrapper(t.update, f, "read")
+    r = requests.put(version['signedUpload'], data=wrapped_file, headers={'Content-Type': 'application/zip'})
 
   print('> Deploying version v' + str(version['number']))
   deploy_version(version['id'])
@@ -158,7 +237,6 @@ def deploy_tf(tf_model, name):
   #os.remove('./model.zip')
 
   print('> TensorFlow Model deployed')
-  return version['number']
 
 # ------ SCIKIT
 def deploy_scikit(scikit_model, name):
@@ -189,9 +267,13 @@ def deploy_scikit(scikit_model, name):
   zipf.close()
 
   # upload to storage
-  print('> Uploading model')
+  model_size = Path('./model.zip').stat().st_size
+  print('> Uploading model ({0})'.format(sizeof_fmt(model_size)), flush=True)
   f = open("./model.zip", 'rb')
-  r = requests.put(version['signedUpload'], data=f, headers={'Content-Type': 'application/zip'})
+  
+  with tqdm(total=model_size, unit="B", unit_scale=True, unit_divisor=1024, ncols=100) as t:
+    wrapped_file = CallbackIOWrapper(t.update, f, "read")
+    r = requests.put(version['signedUpload'], data=wrapped_file, headers={'Content-Type': 'application/zip'})
 
   print('> Deploying version v' + str(version['number']))
   deploy_version(version['id'])
@@ -200,7 +282,6 @@ def deploy_scikit(scikit_model, name):
   os.remove('./model.zip')
 
   print('> Scikit Model deployed')
-  return version['number']
 
 # ------ SPACY
 def deploy_spacy(nlp_model, name):
@@ -231,9 +312,13 @@ def deploy_spacy(nlp_model, name):
   zipf.close()
 
   # upload to storage
-  print('> Uploading model')
+  model_size = Path('./model.zip').stat().st_size
+  print('> Uploading model ({0})'.format(sizeof_fmt(model_size)), flush=True)
   f = open("./model.zip", 'rb')
-  r = requests.put(version['signedUpload'], data=f, headers={'Content-Type': 'application/zip'})
+
+  with tqdm(total=model_size, unit="B", unit_scale=True, unit_divisor=1024, ncols=100) as t:
+    wrapped_file = CallbackIOWrapper(t.update, f, "read")
+    r = requests.put(version['signedUpload'], data=wrapped_file, headers={'Content-Type': 'application/zip'})
 
   print('> Deploying version v' + str(version['number']))
   deploy_version(version['id'])
@@ -241,14 +326,13 @@ def deploy_spacy(nlp_model, name):
   shutil.rmtree('./inferrd-model')
   #os.remove('./model.zip')
 
-  print('> Model deployed')
-  return version['number']
+  print('> spaCy model deployed')
   
 def get_request_history(apiKey, kwargs):
   print('Getting request')
 
 def call_model(serveKey, payload):
-  r = requests.post(api_host + '/infer/' + serveKey + '/predict', data=json.dumps(payload), headers={'Content-Type': 'application/json'})
+  r = requests.post(get_api_host() + '/infer/' + serveKey + '/predict', data=json.dumps(payload), headers={'Content-Type': 'application/json'})
   return r.json()
 
 def get(modelName):
@@ -256,7 +340,13 @@ def get(modelName):
   serveKey = model['key']
 
   def infer(payload):
-    r = requests.post(api_host + '/infer/' + serveKey + '/predict', data=json.dumps(payload), headers={'Content-Type': 'application/json'})
+    headers = {'Content-Type': 'application/json'}
+    
+    # pass in the token when you can, to authenticate requests
+    if(getApiKey() != ''):
+      headers['Authorization'] = 'Token ' + getApiKey()
+
+    r = requests.post(get_api_host() + '/infer/' + serveKey + '/predict', data=json.dumps(payload), headers=headers)
     return r.json()
 
   return infer
\ No newline at end of file
