@@ -10,6 +10,7 @@ except ImportError: # for Python<3.8
     import importlib_metadata as metadata
 
 api_host = 'https://api.inferrd.com'
+#api_host = 'http://localhost:3000'
 
 IN_COLAB = False
 try:
@@ -27,6 +28,11 @@ def zipdir(path, ziph):
             # remove base folder
             ziph.write(filePath, inZipPath.replace(path.split('/')[-1] + '/', ''))
 
+def set_api_host(host):
+  api_host = host
+
+def get_api_host():
+  return api_host
 
 def auth(key):
   '''
@@ -60,12 +66,12 @@ def getApiKey():
 def get_model(name):
   api_key = getApiKey()
 
-  r = requests.get(api_host + '/service/find/' + name, headers={'Content-Type': 'application/json', 'Authorization': 'Token ' + api_key})
+  r = requests.get(get_api_host() + '/service/find/' + name, headers={'Content-Type': 'application/json', 'Authorization': 'Token ' + api_key})
 
   model = r.json()
 
   if 'id' not in model:
-    raise Exception("Model {0} not found in your Inferrd account. Please double check spelling.".format(name))
+    raise Exception("Model \"{0}\" not found in your Inferrd account. Please double check spelling.".format(name))
 
   return model
 
@@ -76,6 +82,53 @@ def new_version(modelId):
 
   return r.json()
 
+def new_job(job_name, team_name, **kwargs):
+  gpu_ram = 2000
+  cpu_ram = 4000
+
+  if 'cpu_ram' in kwargs:
+    cpu_ram = kwargs['cpu_ram']
+
+  if 'gpu_ram' in kwargs:
+    gpu_ram = kwargs['gpu_ram']
+
+  api_key = getApiKey()
+
+  team = get_team(team_name)
+
+  r = requests.post(get_api_host() + '/team/' + team['id'] + '/jobs', json={
+    "name": job_name,
+    "cpuRamMb": cpu_ram,
+    "gpuRamMb": gpu_ram
+  }, headers={'Content-Type': 'application/json', 'Authorization': 'Token ' + api_key})
+
+  job = r.json()
+
+  if 'error' in job:
+    raise Exception(job['message'])
+
+  return job
+
+def run_job(jobId):
+  api_key = getApiKey()
+  r = requests.post(get_api_host() + '/job/' + jobId + '/run', headers={'Content-Type': 'application/json', 'Authorization': 'Token ' + api_key})
+  job = r.json()
+
+  if 'error' in job:
+    raise Exception(job['message'])
+
+def get_team(name):
+  api_key = getApiKey()
+
+  r = requests.get(get_api_host() + '/team/find/' + name, headers={'Content-Type': 'application/json', 'Authorization': 'Token ' + api_key})
+  team = r.json()
+
+  if 'id' not in team:
+    raise Exception("Team \"{0}\" not found in your Inferrd account. Please double check spelling.".format(name))
+
+  return team
+
+
 def deploy_version(versionId, **kwargs):
   api_key = getApiKey()
 
